@@ -1,5 +1,5 @@
 /*
- * $Id: bookmarks.c,v 1.27 2005/01/16 16:48:00 tal197 Exp $
+ * $Id: bookmarks.c,v 1.31 2005/08/01 07:10:18 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2005, the ROX-Filer team.
@@ -38,6 +38,8 @@
 #include "main.h"
 #include "mount.h"
 #include "action.h"
+#include "options.h"
+#include "bind.h"
 
 static GList *history = NULL;		/* Most recent first */
 static GList *history_tail = NULL;	/* Oldest item */
@@ -88,6 +90,7 @@ void bookmarks_show_menu(FilerWindow *filer_window)
 	if (event->type == GDK_BUTTON_RELEASE ||
 	    event->type == GDK_BUTTON_PRESS)
 		button = ((GdkEventButton *) event)->button;
+	gdk_event_free(event);
 
 	menu = GTK_MENU(bookmarks_build_menu(filer_window));
 	gtk_menu_popup(menu, NULL, NULL, position_menu, filer_window,
@@ -342,7 +345,7 @@ static void update_bookmarks()
 	gchar *path;
 
 	/* Update the bookmarks, if possible */
-	path = choices_find_path_load("Bookmarks.xml", PROJECT);
+	path = choices_find_xdg_path_load("Bookmarks.xml", PROJECT, SITE);
 	if (path)
 	{
 		XMLwrapper *wrapper;
@@ -400,7 +403,8 @@ static void bookmarks_save()
 {
 	guchar	*save_path;
 
-	save_path = choices_find_path_save("Bookmarks.xml", PROJECT, TRUE);
+	save_path = choices_find_xdg_path_save("Bookmarks.xml", PROJECT, SITE,
+					       TRUE);
 	if (save_path)
 	{
 		save_xml_file(bookmarks->doc, save_path);
@@ -440,6 +444,8 @@ static void bookmarks_activate(GtkMenuShell *item, FilerWindow *filer_window)
 {
 	const gchar *mark;
 	GtkLabel *label;
+	GdkEvent *event;
+	gboolean new_win=FALSE;
 
 	mark=g_object_get_data(G_OBJECT(item), "bookmark-path");
 	if(!mark) {
@@ -447,15 +453,34 @@ static void bookmarks_activate(GtkMenuShell *item, FilerWindow *filer_window)
 		mark = gtk_label_get_text(label);
 	}
 
+	event=gtk_get_current_event();
+	if(event)
+	{
+		if(event->type==GDK_BUTTON_PRESS ||
+		   event->type==GDK_BUTTON_RELEASE)
+		{
+			GdkEventButton *button=(GdkEventButton *) event;
+
+			new_win=o_new_button_1.int_value?
+				button->button==1: button->button!=1;
+		}
+		gdk_event_free(event);
+	}
+
 	if (strcmp(mark, filer_window->sym_path) != 0)
-		filer_change_to(filer_window, mark, NULL);
+	{
+		if(new_win)
+			filer_opendir(mark, filer_window, NULL);
+		else
+			filer_change_to(filer_window, mark, NULL);
+	}
 	if (g_hash_table_lookup(fstab_mounts, filer_window->real_path) &&
 		!mount_is_mounted(filer_window->real_path, NULL, NULL))
 	{
 		GList	*paths;
 
 		paths = g_list_prepend(NULL, filer_window->real_path);
-		action_mount(paths, FALSE, -1);
+		action_mount(paths, FALSE, TRUE, -1);
 		g_list_free(paths);
 	}
 }
