@@ -1,5 +1,5 @@
 /*
- * $Id: dnd.c,v 1.134 2005/01/16 16:43:04 tal197 Exp $
+ * $Id: dnd.c,v 1.138 2005/08/01 07:10:18 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2005, the ROX-Filer team.
@@ -114,7 +114,8 @@ static void prompt_action(GList *paths, gchar *dest);
 typedef enum {
 	MENU_COPY,
 	MENU_MOVE,
-	MENU_LINK,
+	MENU_LINK_REL,
+	MENU_LINK_ABS,
 } MenuActionType;
 
 #undef N_
@@ -122,7 +123,8 @@ typedef enum {
 static GtkItemFactoryEntry menu_def[] = {
 {N_("Copy"),		NULL, menuitem_response, MENU_COPY, 	NULL},
 {N_("Move"),		NULL, menuitem_response, MENU_MOVE, 	NULL},
-{N_("Link"),		NULL, menuitem_response, MENU_LINK, 	NULL},
+{N_("Link (relative)"),	NULL, menuitem_response, MENU_LINK_REL, NULL},
+{N_("Link (absolute)"),	NULL, menuitem_response, MENU_LINK_ABS,	NULL},
 };
 static GtkWidget *dnd_menu = NULL;
 
@@ -138,7 +140,7 @@ GdkAtom XdndDirectSave0;
 GdkAtom xa_text_plain;
 GdkAtom text_uri_list;
 GdkAtom text_x_moz_url;
-GdkAtom application_octet_stream;
+GdkAtom xa_application_octet_stream;
 GdkAtom xa_string; /* Not actually used for DnD, but the others are here! */
 
 int spring_in_progress = 0;	/* Non-zero changes filer_opendir slightly */
@@ -156,7 +158,7 @@ void dnd_init(void)
 	xa_text_plain = gdk_atom_intern("text/plain", FALSE);
 	text_uri_list = gdk_atom_intern("text/uri-list", FALSE);
 	text_x_moz_url = gdk_atom_intern("text/x-moz-url", FALSE);
-	application_octet_stream = gdk_atom_intern("application/octet-stream",
+	xa_application_octet_stream = gdk_atom_intern("application/octet-stream",
 			FALSE);
 	xa_string = gdk_atom_intern("STRING", FALSE);
 
@@ -495,8 +497,7 @@ const guchar *dnd_motion_item(GdkDragContext *context, DirItem **item_p)
 		 * executable file then act as though the drop is to the
 		 * window background.
 		 */
-		if (item->base_type != TYPE_DIRECTORY
-				&& !(item->mime_type == application_executable))
+		if (item->base_type != TYPE_DIRECTORY && !EXECUTABLE_FILE(item))
 		{
 			item = NULL;
 			*item_p = NULL;
@@ -525,7 +526,7 @@ const guchar *dnd_motion_item(GdkDragContext *context, DirItem **item_p)
 	{
 		if (provides(context, text_uri_list) ||
 				provides(context, text_x_moz_url) ||
-				provides(context, application_octet_stream))
+				provides(context, xa_application_octet_stream))
 			return drop_dest_prog;
 	}
 
@@ -609,8 +610,8 @@ static gboolean drag_drop(GtkWidget 	  *widget,
 		target = text_uri_list;
 	else if (provides(context, text_x_moz_url))
 		target = text_x_moz_url;
-	else if (provides(context, application_octet_stream))
-		target = application_octet_stream;
+	else if (provides(context, xa_application_octet_stream))
+		target = xa_application_octet_stream;
 	else
 	{
 		if (dest_type == drop_dest_dir)
@@ -787,12 +788,12 @@ static void got_data_xds_reply(GtkWidget 		*widget,
 		/* Sender couldn't save there - ask for another
 		 * type if possible.
 		 */
-		if (provides(context, application_octet_stream))
+		if (provides(context, xa_application_octet_stream))
 		{
 			mark_unsafe = FALSE;	/* Wait and see */
 
 			gtk_drag_get_data(widget, context,
-					application_octet_stream, time);
+					xa_application_octet_stream, time);
 		}
 		else
 			error = _("Remote app can't or won't send me "
@@ -966,7 +967,7 @@ static void got_uri_list(GtkWidget 		*widget,
 		 * machine. Get it via the X server if possible.
 		 */
 
-		if (provides(context, application_octet_stream))
+		if (provides(context, xa_application_octet_stream))
 		{
 			char	*leaf;
 			leaf = strrchr(uri_list->data, '/');
@@ -977,7 +978,7 @@ static void got_uri_list(GtkWidget 		*widget,
 			g_dataset_set_data_full(context, "leafname",
 				unescape_uri((EscapedPath *) leaf), g_free);
 			gtk_drag_get_data(widget, context,
-					application_octet_stream, time);
+					xa_application_octet_stream, time);
 			send_reply = FALSE;
 		}
 		else if ((strncasecmp(uri_list->data, "http:", 5) == 0) ||
@@ -1030,7 +1031,7 @@ static void got_uri_list(GtkWidget 		*widget,
 		else if (context->action == GDK_ACTION_COPY)
 			action_copy(local_paths, dest_path, NULL, -1);
 		else if (context->action == GDK_ACTION_LINK)
-			action_link(local_paths, dest_path, NULL);
+			action_link(local_paths, dest_path, NULL, TRUE);
 		else
 			error = _("Unknown action requested");
 
@@ -1055,8 +1056,10 @@ static void menuitem_response(gpointer data, guint action, GtkWidget *widget)
 		action_move(prompt_local_paths, prompt_dest_path, NULL, -1);
 	else if (action == MENU_COPY)
 		action_copy(prompt_local_paths, prompt_dest_path, NULL, -1);
-	else if (action == MENU_LINK)
-		action_link(prompt_local_paths, prompt_dest_path, NULL);
+	else if (action == MENU_LINK_REL)
+		action_link(prompt_local_paths, prompt_dest_path, NULL, TRUE);
+	else if (action == MENU_LINK_ABS)
+		action_link(prompt_local_paths, prompt_dest_path, NULL, FALSE);
 } 
 
 /* When some local files are dropped somewhere with ACTION_ASK, this
@@ -1065,6 +1068,7 @@ static void menuitem_response(gpointer data, guint action, GtkWidget *widget)
 static void prompt_action(GList *paths, gchar *dest)
 {
 	GList		*next;
+	GdkEvent	*event;
 
 	if (prompt_local_paths)
 	{
@@ -1091,7 +1095,10 @@ static void prompt_action(GList *paths, gchar *dest)
 	/* Shade 'Set Icon' if there are multiple files */
 	menu_set_items_shaded(dnd_menu, g_list_length(paths) != 1, 4, 1);
 
-	show_popup_menu(dnd_menu, gtk_get_current_event(), 1);
+	event = gtk_get_current_event();
+	show_popup_menu(dnd_menu, event, 1);
+	if (event)
+		gdk_event_free(event);
 }
 
 
