@@ -1,5 +1,5 @@
 /*
- * $Id: main.c,v 1.176 2005/01/16 16:48:01 tal197 Exp $
+ * $Id: main.c,v 1.196 2005/11/21 20:05:55 kerofin Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2005, the ROX-Filer team.
@@ -115,7 +115,8 @@ GtkTooltips *tooltips = NULL;
 #define HELP N_("Usage: ROX-Filer/AppRun [OPTION]... [FILE]...\n"	\
        "Open each directory or file listed, or the current working\n"	\
        "directory if no arguments are given.\n\n"			\
-       "  -b, --bottom=PANEL	open PAN as a bottom-edge panel\n"	\
+       "  -b, --border=PANEL	open PANEL as a border panel\n"	\
+       "  -B, --bottom=PANEL	open PAN as a bottom-edge panel\n"	\
        "  -c, --client-id=ID	used for session management\n"		\
        "  -d, --dir=DIR		open DIR as directory (not application)\n"  \
        "  -D, --close=DIR	close DIR and its subdirectories\n"     \
@@ -127,6 +128,7 @@ GtkTooltips *tooltips = NULL;
        "  -r, --right=PANEL	open PAN as a right-edge panel\n"	\
        "  -R, --RPC		invoke method call read from stdin\n"	\
        "  -s, --show=FILE	open a directory showing FILE\n"	\
+       "  -S, --rox-session	use default panel and pinboard options, and -n\n"\
        "  -t, --top=PANEL	open PANEL as a top-edge panel\n"	\
        "  -u, --user		show user name in each window \n"	\
        "  -v, --version		display the version information and exit\n"   \
@@ -134,14 +136,15 @@ GtkTooltips *tooltips = NULL;
        "\nReport bugs to " BUGS_TO ".\n"		\
        "Home page (including updated versions): http://rox.sourceforge.net/\n")
 
-#define SHORT_OPS "c:d:t:b:l:r:op:s:hvnux:m:D:R"
+#define SHORT_OPS "c:d:t:b:l:r:B:op:s:hvnux:m:D:RS"
 
 #ifdef HAVE_GETOPT_LONG
 static struct option long_opts[] =
 {
 	{"dir", 1, NULL, 'd'},
 	{"top", 1, NULL, 't'},
-	{"bottom", 1, NULL, 'b'},
+	{"bottom", 1, NULL, 'B'},
+	{"border", 1, NULL, 'b'},
 	{"left", 1, NULL, 'l'},
 	{"override", 0, NULL, 'o'},
 	{"pinboard", 1, NULL, 'p'},
@@ -152,6 +155,7 @@ static struct option long_opts[] =
 	{"new", 0, NULL, 'n'},
 	{"RPC", 0, NULL, 'R'},
 	{"show", 1, NULL, 's'},
+	{"rox-session", 0, NULL, 'S'},
 	{"examine", 1, NULL, 'x'},
 	{"close", 1, NULL, 'D'},
 	{"mime-type", 1, NULL, 'm'},
@@ -163,6 +167,16 @@ static struct option long_opts[] =
 /* Take control of panels away from WM? */
 Option o_override_redirect;
 
+/* Options used when we are called by ROX-Session */
+enum {
+	SESSION_PANEL_ONLY,
+	SESSION_PINBOARD_ONLY,
+	SESSION_BOTH,
+};
+Option o_session_panel_or_pin;
+Option o_session_panel_name;
+Option o_session_pinboard_name;
+
 /* Always start a new filer, even if one seems to be already running */
 gboolean new_copy = FALSE;
 
@@ -182,6 +196,8 @@ static void child_died(int signum);
 static void child_died_callback(void);
 static void wake_up_cb(gpointer data, gint source, GdkInputCondition condition);
 static void xrandr_size_change(GdkScreen *screen, gpointer user_data);
+static void add_default_panel_and_pinboard(xmlNodePtr body);
+static GList *build_launch(Option *option, xmlNode *node, guchar *label);
 
 /****************************************************************
  *			EXTERNAL INTERFACE			*
@@ -192,27 +208,37 @@ static void xrandr_size_change(GdkScreen *screen, gpointer user_data);
 
 static int rox_x_error(Display *display, XErrorEvent *error)
 {
-	  gchar buf[64];
-          
-	  XGetErrorText(display, error->error_code, buf, 63);
+	gchar buf[64];
 
-	  g_warning ("The program '%s' received an X Window System error.\n"
-                             "This probably reflects a bug in the program.\n"
-                             "The error was '%s'.\n"
-                             "  (Details: serial %ld error_code %d request_code %d minor_code %d)\n"
-                             "  (Note to programmers: normally, X errors are reported asynchronously;\n"
-                             "   that is, you will receive the error a while after causing it.\n"
-                             "   To debug your program, run it with the --sync command line\n"
-                             "   option to change this behavior. You can then get a meaningful\n"
-                             "   backtrace from your debugger.)",
-                             g_get_prgname (),
-                             buf,
-                             error->serial, 
-                             error->error_code, 
-                             error->request_code,
-                             error->minor_code);
-          
-	  abort();
+	XGetErrorText(display, error->error_code, buf, 63);
+
+	g_warning ("The program '%s' received an X Window System error.\n"
+			"This probably reflects a bug in the program.\n"
+			"The error was '%s'.\n"
+			"  (Details: serial %ld error_code %d request_code %d minor_code %d)\n"
+			"  (Note to programmers: normally, X errors are reported asynchronously;\n"
+			"   that is, you will receive the error a while after causing it.\n"
+			"   To debug your program, run it with the --sync command line\n"
+			"   option to change this behavior. You can then get a meaningful\n"
+			"   backtrace from your debugger.)",
+			g_get_prgname (),
+			buf,
+			error->serial, 
+			error->error_code, 
+			error->request_code,
+			error->minor_code);
+
+	/* Try to cope with BadWindow errors */
+	if (error->error_code == BadWindow || error->error_code == BadDrawable)
+	{
+		g_warning(_("We got a BadWindow error from the X server. "
+			    "This might be due to this GTK bug (during drag-and-drop?):\n"
+			    "http://bugzilla.gnome.org/show_bug.cgi?id=152151\n"
+			    "Trying to continue..."));
+		return 0;
+	}
+
+	abort();
 }
 
 /* Parses the command-line to work out what the user wants to do.
@@ -230,6 +256,21 @@ int main(int argc, char **argv)
 	gboolean	show_user = FALSE;
 	xmlDocPtr	rpc, soap_rpc = NULL, reply;
 	xmlNodePtr	body;
+	int		fd, ofd0=-1;
+
+	/* Relocate stdin. We do need it (-R), but it can cause problems if
+	 * a child process wants a password, etc...
+	 * Do this BEFORE opening anything (e.g., the X connection), in
+	 * case fd 0 isn't open at this point.
+	 */
+	fd = open("/dev/null", O_RDONLY);
+	if (fd > 0)
+	{
+		ofd0=dup(0);
+		close(0);
+		dup2(fd, 0);
+		close(fd);
+	}
 
 	home_dir = g_get_home_dir();
 	home_dir_len = strlen(home_dir);
@@ -241,12 +282,7 @@ int main(int argc, char **argv)
 	choices_init();
 	options_init();
 	i18n_init();
-	xtype_init();
-
-#ifdef HAVE_GNOME_VFS
-	if (!gnome_vfs_init())
-		g_warning("Failed to init gnomevfs!");
-#endif
+	xattr_init();
 
 	if (!app_dir)
 	{
@@ -294,6 +330,14 @@ int main(int argc, char **argv)
 
 	option_add_int(&o_override_redirect, "override_redirect", FALSE);
 
+	option_add_int(&o_session_panel_or_pin, "session_panel_or_pin",
+		       SESSION_BOTH);
+	option_add_string(&o_session_panel_name, "session_panel_name",
+			  "Default");
+	option_add_string(&o_session_pinboard_name, "session_pinboard_name",
+			  "Default");
+	option_register_widget("launch", build_launch);
+
 	/* The idea here is to convert the command-line arguments
 	 * into a SOAP RPC.
 	 * We attempt to invoke the call on an already-running copy of
@@ -392,15 +436,25 @@ int main(int argc, char **argv)
 			case 'l':
 			case 'r':
 			case 't':
-			case 'b':
+			case 'B':
 				/* Argument is a leaf (or starts with /) */
 				soap_add(body, "Panel", "Name", VALUE,
 					 "Side", c == 'l' ? "Left" :
 						 c == 'r' ? "Right" :
 						 c == 't' ? "Top" :
-						 c == 'b' ? "Bottom" :
+						 c == 'B' ? "Bottom" :
 						 "Unkown");
 				break;
+			case 'b':
+				/* Argument is a leaf (or starts with /) */
+				if (*VALUE)
+					soap_add(body, "Panel", "Name", VALUE,
+							NULL, NULL);
+				else
+					soap_add(body, "Panel",
+							"Side", "Bottom",
+							NULL, NULL);
+				break;
 			case 'p':
 				soap_add(body, "Pinboard",
 						"Name", VALUE, NULL, NULL);
@@ -423,10 +477,30 @@ int main(int argc, char **argv)
 				client_id = g_strdup(VALUE);
 				break;
 			case 'R':
+				/* Reconnect stdin */
+				if(ofd0>-1) {
+					close(0);
+					dup2(ofd0, 0);
+				}
 				soap_rpc = xmlParseFile("-");
 				if (!soap_rpc)
 					g_error("Invalid XML in RPC");
+				/* Disconnect stdin again */
+				fd = open("/dev/null", O_RDONLY);
+				if (fd > 0)
+				{
+					close(0);
+					dup2(fd, 0);
+					close(fd);
+				}
+				
 				break;
+
+		        case 'S':
+				new_copy = TRUE;
+				add_default_panel_and_pinboard(body);
+				break;
+
 			default:
 				printf(_(USAGE));
 				return EXIT_FAILURE;
@@ -485,7 +559,6 @@ int main(int argc, char **argv)
 	 */
 	if (!new_copy)
 	{
-		int fd;
 		pid_t child;
 
 		child = fork();
@@ -494,17 +567,6 @@ int main(int argc, char **argv)
 		/* Otherwise we're the child (or an error occurred - ignore
 		 * it!).
 		 */
-
-		/* Close stdin. We don't need it, and it can cause problems if
-		 * a child process wants a password, etc...
-		 */
-		fd = open("/dev/null", O_RDONLY);
-		if (fd > 0)
-		{
-			close(0);
-			dup2(fd, 0);
-			close(fd);
-		}
 	}
 
 	/* Initialize the rest of the filer... */
@@ -560,6 +622,9 @@ int main(int argc, char **argv)
 	session_init(client_id);
 	g_free(client_id);
 
+	/* See if we need to migrate the Choices directories*/
+	choices_migrate();
+
 	/* Finally, execute the request */
 	reply = run_soap(rpc);
 	xmlFreeDoc(rpc);
@@ -623,13 +688,6 @@ static void show_features(void)
 		_("No")
 #endif
 		);
-	g_print("%s... %s\n", _("GNOME-VFS library"),
-# ifdef HAVE_GNOME_VFS
-			_("Yes")
-# else
-			_("No (need 2.8.0 or later)")
-# endif
-		  );
 	g_print("%s... %s\n", _("Dnotify support"),
 #ifdef USE_DNOTIFY
 		_("Yes")
@@ -644,6 +702,9 @@ static void show_features(void)
 		_("No (apsymbols.h not found)")
 #endif
 	       );
+
+	g_print("%s... %s\n", _("Extended attribute support"),
+		xattr_supported(NULL)? _("Yes"): _("No"));
 }
 
 static void soap_add(xmlNodePtr body,
@@ -717,7 +778,7 @@ static void wake_up_cb(gpointer data, gint source, GdkInputCondition condition)
 	if (child_died_flag)
 		child_died_callback();
 #ifdef USE_DNOTIFY
-	if (dnotify_wakeup)
+	if (dnotify_wakeup_flag)
 		dnotify_wakeup();
 #endif
 }
@@ -729,3 +790,63 @@ static void xrandr_size_change(GdkScreen *screen, gpointer user_data)
 	panel_update_size();
 	pinboard_update_size();
 }
+
+static void add_default_panel_and_pinboard(xmlNodePtr body)
+{
+	char *name;
+
+	if (o_session_panel_or_pin.int_value != SESSION_PANEL_ONLY)
+	{
+		name=o_session_pinboard_name.value;
+		if (!name[0])
+			name="Default";
+		soap_add(body, "Pinboard","Name", name, NULL, NULL);
+	}
+					
+	if (o_session_panel_or_pin.int_value != SESSION_PINBOARD_ONLY)
+	{
+		name = o_session_panel_name.value;
+		if (!name[0])
+			name="Default";
+		
+		soap_add(body, "Panel", "Name", name, NULL, NULL);
+	}
+}
+
+static GtkWidget *launch_button_new(const char *label, const char *uri)
+{
+	GtkWidget *button;
+	GClosure *closure;
+
+	button = button_new_mixed(GTK_STOCK_PREFERENCES, label);
+	closure = g_cclosure_new_swap(G_CALLBACK(launch_uri),
+					g_strdup(uri),
+					(GClosureNotify) g_free);
+	g_signal_connect_closure(button, "clicked", closure, FALSE);
+
+	allow_right_click(button);
+
+	return button;
+}
+
+static GList *build_launch(Option *option, xmlNode *node, guchar *label)
+{
+	GtkWidget *align;
+	char *uri;
+
+	g_return_val_if_fail(option == NULL, NULL);
+	g_return_val_if_fail(label != NULL, NULL);
+
+	uri = xmlGetProp(node, "uri");
+
+	g_return_val_if_fail(uri != NULL, NULL);
+
+	align = gtk_alignment_new(0, 0.5, 0, 0);
+
+	gtk_container_add(GTK_CONTAINER(align),
+			launch_button_new(_(label), uri));
+
+	g_free(uri);
+
+	return g_list_append(NULL, align);
+}
