@@ -1,5 +1,5 @@
 /*
- * $Id: options.c,v 1.93 2005/01/16 16:48:01 tal197 Exp $
+ * $Id: options.c,v 1.95 2005/10/02 16:48:30 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2005 Thomas Leonard
@@ -170,7 +170,7 @@ void options_init(void)
 	option_hash = g_hash_table_new(g_str_hash, g_str_equal);
 	widget_builder = g_hash_table_new(g_str_hash, g_str_equal);
 
-	path = choices_find_path_load("Options", PROJECT);
+	path = choices_find_xdg_path_load("Options", PROJECT, SITE);
 	if (path)
 	{
 		/* Load in all the options set in the filer, storing them
@@ -659,9 +659,9 @@ static GtkWidget *build_radio(xmlNode *radio, GtkWidget *prev)
 	return button;
 }
 
-static void build_menu_item(xmlNode *node, GtkWidget *option_menu)
+static void build_menu_item(xmlNode *node, GtkWidget *menu)
 {
-	GtkWidget	*item, *menu;
+	GtkWidget	*item;
 	guchar		*label;
 
 	g_return_if_fail(strcmp(node->name, "item") == 0);
@@ -670,9 +670,8 @@ static void build_menu_item(xmlNode *node, GtkWidget *option_menu)
 	item = gtk_menu_item_new_with_label(_(label));
 	g_free(label);
 
-	menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(option_menu));
+	gtk_widget_show(item);
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-	gtk_widget_show_all(menu);
 
 	g_object_set_data(G_OBJECT(item), "value", xmlGetProp(node, "value"));
 }
@@ -977,7 +976,7 @@ static GtkWidget *build_window_frame(GtkTreeView **tree_view)
 	gtk_widget_grab_default(button);
 	gtk_widget_grab_focus(button);
 
-	save_path = choices_find_path_save("...", PROJECT, FALSE);
+	save_path = choices_find_xdg_path_save("...", PROJECT, SITE, FALSE);
 	if (save_path)
 	{
 		string = g_strdup_printf(_("Choices will be saved as:\n%s"),
@@ -1148,7 +1147,7 @@ static void save_options(void)
 	GList	*next;
 	guchar	*save, *save_new;
 
-	save = choices_find_path_save("Options", PROJECT, TRUE);
+	save = choices_find_xdg_path_save("Options", PROJECT, SITE, TRUE);
 	if (!save)
 		goto out;
 
@@ -1667,16 +1666,17 @@ static GList *build_menu(Option *option, xmlNode *node, guchar *label)
 	gtk_box_pack_start(GTK_BOX(hbox), option_menu, FALSE, TRUE, 0);
 
 	om = gtk_menu_new();
-	gtk_option_menu_set_menu(GTK_OPTION_MENU(option_menu), om);
-
-	add_to_size_group(node, option_menu);
 
 	for (item = node->xmlChildrenNode; item; item = item->next)
 	{
 		if (item->type == XML_ELEMENT_NODE)
-			build_menu_item(item, option_menu);
+			build_menu_item(item, om);
 	}
 
+	gtk_widget_show(om);
+	gtk_option_menu_set_menu(GTK_OPTION_MENU(option_menu), om);
+	add_to_size_group(node, option_menu);
+
 	option->update_widget = update_menu;
 	option->read_widget = read_menu;
 	option->widget = option_menu;
