@@ -1,5 +1,5 @@
 /*
- * $Id: pixmaps.c,v 1.120 2005/01/16 16:48:02 tal197 Exp $
+ * $Id: pixmaps.c,v 1.125 2005/11/13 11:24:41 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2005, the ROX-Filer team.
@@ -30,6 +30,7 @@
 
 #define PIXMAP_PURGE_TIME 1200
 #define PIXMAP_THUMB_SIZE  128
+#define PIXMAP_THUMB_TOO_OLD_TIME  5
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -57,6 +58,7 @@
 #include "type.h"
 
 GFSCache *pixmap_cache = NULL;
+GFSCache *desktop_icon_cache = NULL;
 
 static const char * bad_xpm[] = {
 "12 12 3 1",
@@ -111,6 +113,7 @@ static GtkIconSize mount_icon_size = -1;
 static void load_default_pixmaps(void);
 static gint purge(gpointer data);
 static MaskedPixmap *image_from_file(const char *path);
+static MaskedPixmap *image_from_desktop_file(const char *path);
 static MaskedPixmap *get_bad_image(void);
 static GdkPixbuf *scale_pixbuf_up(GdkPixbuf *src, int max_w, int max_h);
 static GdkPixbuf *get_thumbnail_for(const char *path);
@@ -134,6 +137,7 @@ void pixmaps_init(void)
 	gtk_widget_push_colormap(gdk_rgb_get_colormap());
 
 	pixmap_cache = g_fscache_new((GFSLoadFunc) image_from_file, NULL, NULL);
+	desktop_icon_cache = g_fscache_new((GFSLoadFunc) image_from_desktop_file, NULL, NULL);
 
 	g_timeout_add(10000, purge, NULL);
 
@@ -336,6 +340,7 @@ void pixmap_background_thumb(const gchar *path, GFunc callback, gpointer data)
 		gdk_pixbuf_unref(pixbuf);
 		g_fscache_insert(pixmap_cache, path, image, TRUE);
 		callback(data, (gchar *) path);
+		g_object_unref(G_OBJECT(image));
 		return;
 	}
 
@@ -381,7 +386,7 @@ void pixmap_background_thumb(const gchar *path, GFunc callback, gpointer data)
 			if (item->flags & ITEM_FLAG_APPDIR)
 				thumb_prog = g_strconcat(thumb_prog, "/AppRun",
 						       NULL);
-						  
+
 			execl(thumb_prog, thumb_prog, path,
 			      thumbnail_path(path),
 			      g_strdup_printf("%d", PIXMAP_THUMB_SIZE),
@@ -523,14 +528,15 @@ static gchar *thumbnail_program(MIME_type *type)
 		return NULL;
 
 	leaf = g_strconcat(type->media_type, "_", type->subtype, NULL);
-	path = choices_find_path_load(leaf, "MIME-thumb");
+	path = choices_find_xdg_path_load(leaf, "MIME-thumb", SITE);
 	g_free(leaf);
 	if (path)
 	{
 		return path;
 	}
 
-	path = choices_find_path_load(type->media_type, "MIME-thumb");
+	path = choices_find_xdg_path_load(type->media_type, "MIME-thumb",
+					  SITE);
 
 	return path;
 }
@@ -586,6 +592,7 @@ static GdkPixbuf *get_thumbnail_for(const char *pathname)
 	char *thumb_path, *md5, *uri, *path;
 	const char *ssize, *smtime;
 	struct stat info;
+	time_t ttime, now;
 
 	path = pathdup(pathname);
 	uri = g_filename_to_uri(path, NULL, NULL);
@@ -614,7 +621,12 @@ static GdkPixbuf *get_thumbnail_for(const char *pathname)
 	if (mc_stat(path, &info) != 0)
 		goto err;
 
-	if (info.st_mtime != atol(smtime) || info.st_size != atol(ssize))
+	ttime=(time_t) atol(smtime);
+	time(&now);
+	if (info.st_mtime != ttime && now>ttime+PIXMAP_THUMB_TOO_OLD_TIME)
+		goto err;
+
+	if (info.st_size < atol(ssize))
 		goto err;
 
 	goto out;
@@ -653,6 +665,55 @@ static MaskedPixmap *image_from_file(const char *path)
 	return image;
 }
 
+/* Load this icon named by this .desktop file from the current theme.
+ * NULL on failure.
+ */
+static MaskedPixmap *image_from_desktop_file(const char *path)
+{
+	GError *error = NULL;
+	MaskedPixmap *image = NULL;
+	char *icon = NULL;
+
+	icon = get_value_from_desktop_file(path,
+					"Desktop Entry", "Icon", &error);
+	if (error)
+	{
+		g_warning("Failed to parse .desktop file '%s':\n%s",
+				path, error->message);
+		goto err;
+	}
+	if (!icon)
+		goto err;
+
+	if (icon[0] == '/')
+		image = image_from_file(icon);
+	else
+	{
+		GdkPixbuf *pixbuf;
+		int tmp_fd;
+
+		/* SVG reader is very noisy, so redirect stderr to stdout */
+		tmp_fd = dup(2);
+		dup2(1, 2);
+		pixbuf = gtk_icon_theme_load_icon(icon_theme, icon, HUGE_WIDTH,
+							0, NULL);
+		dup2(tmp_fd, 2);
+		close(tmp_fd);
+
+		if (pixbuf == NULL)
+			goto err;	/* Might just not be in the theme */
+
+		image = masked_pixmap_new(pixbuf);
+		g_object_unref(pixbuf);
+	}
+err:
+	if (error != NULL)
+		g_error_free(error);
+	if (icon != NULL)
+		g_free(icon);
+	return image;
+}
+
 /* Scale src down to fit in max_w, max_h and return the new pixbuf.
  * If src is small enough, then ref it and return that.
  */
