@@ -1,5 +1,5 @@
 /*
- * $Id: toolbar.c,v 1.80 2005/01/16 16:48:03 tal197 Exp $
+ * $Id: toolbar.c,v 1.82 2005/08/01 07:10:20 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2005, the ROX-Filer team.
@@ -61,7 +61,6 @@ struct _Tool {
 Option o_toolbar, o_toolbar_info, o_toolbar_disable;
 Option o_toolbar_min_width;
 
-static GtkTooltips *tooltips = NULL;
 static FilerWindow *filer_window_being_counted;
 
 /* TRUE if the button presses (or released) should open a new window,
@@ -176,8 +175,6 @@ void toolbar_init(void)
 	option_add_int(&o_toolbar_min_width, "toolbar_min_width", 1);
 	option_add_notify(option_notify);
 	
-	tooltips = gtk_tooltips_new();
-
 	option_register_widget("tool-options", build_tool_options);
 }
 
@@ -300,6 +297,7 @@ static void toolbar_help_clicked(GtkWidget *widget, FilerWindow *filer_window)
 		menu_rox_help(NULL, HELP_MANUAL, NULL);
 	else
 		filer_opendir(make_path(app_dir, "Help"), NULL, NULL);
+	gdk_event_free(event);
 }
 
 static void toolbar_refresh_clicked(GtkWidget *widget,
@@ -315,6 +313,7 @@ static void toolbar_refresh_clicked(GtkWidget *widget,
 	}
 	else
 		filer_refresh(filer_window);
+	gdk_event_free(event);
 }
 
 static void toolbar_home_clicked(GtkWidget *widget, FilerWindow *filer_window)
@@ -328,6 +327,7 @@ static void toolbar_home_clicked(GtkWidget *widget, FilerWindow *filer_window)
 	}
 	else
 		filer_change_to(filer_window, home_dir, NULL);
+	gdk_event_free(event);
 }
 
 static void toolbar_bookmarks_clicked(GtkWidget *widget,
@@ -348,6 +348,7 @@ static void toolbar_bookmarks_clicked(GtkWidget *widget,
 	{
 		bookmarks_edit();
 	}
+	gdk_event_free(event);
 }
 
 static void toolbar_close_clicked(GtkWidget *widget, FilerWindow *filer_window)
@@ -364,6 +365,7 @@ static void toolbar_close_clicked(GtkWidget *widget, FilerWindow *filer_window)
 	}
 	else if (!filer_window_delete(filer_window->window, NULL, filer_window))
 		gtk_widget_destroy(filer_window->window);
+	gdk_event_free(event);
 }
 
 static void toolbar_up_clicked(GtkWidget *widget, FilerWindow *filer_window)
@@ -377,6 +379,7 @@ static void toolbar_up_clicked(GtkWidget *widget, FilerWindow *filer_window)
 	}
 	else
 		change_to_parent(filer_window);
+	gdk_event_free(event);
 }
 
 static void toolbar_autosize_clicked(GtkWidget *widget, FilerWindow *filer_window)
@@ -384,11 +387,12 @@ static void toolbar_autosize_clicked(GtkWidget *widget, FilerWindow *filer_windo
 	GdkEventButton	*bev;
 
 	bev = (GdkEventButton *) gtk_get_current_event();
-	if (bev->type != GDK_BUTTON_RELEASE)
-		return;
-
-	display_set_layout(filer_window, AUTO_SIZE_ICONS, filer_window->details_type,
-			   TRUE);
+	if (bev->type == GDK_BUTTON_RELEASE)
+	{
+		display_set_layout(filer_window, AUTO_SIZE_ICONS, filer_window->details_type,
+				TRUE);
+	}
+	gdk_event_free((GdkEvent *) bev);
 }
 
 static void toolbar_size_clicked(GtkWidget *widget, FilerWindow *filer_window)
@@ -396,10 +400,9 @@ static void toolbar_size_clicked(GtkWidget *widget, FilerWindow *filer_window)
 	GdkEventButton	*bev;
 
 	bev = (GdkEventButton *) gtk_get_current_event();
-	if (bev->type != GDK_BUTTON_RELEASE)
-		return;
-
-	display_change_size(filer_window, bev->button == 1);
+	if (bev->type == GDK_BUTTON_RELEASE)
+		display_change_size(filer_window, bev->button == 1);
+	gdk_event_free((GdkEvent *) bev);
 }
 
 static void toolbar_sort_clicked(GtkWidget *widget,
@@ -422,6 +425,7 @@ static void toolbar_sort_clicked(GtkWidget *widget,
 
 	bev = (GdkEventButton *) gtk_get_current_event();
 	adjust = (bev->button != 1) && bev->type == GDK_BUTTON_RELEASE;
+	gdk_event_free((GdkEvent *) bev);
 
 	current = -1;
 	dir = filer_window->sort_order;
@@ -490,6 +494,7 @@ static void toolbar_select_clicked(GtkWidget *widget, FilerWindow *filer_window)
 				       filer_window->view);
 	}
 	filer_window->temp_item_selected = FALSE;
+	gdk_event_free(event);
 }
 
 /* If filer_window is NULL, the toolbar is for the options window */
